#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization module for tpRenamer
"""

from __future__ import print_function, division, absolute_import

import os
import inspect

from tpPyUtils import importer
from tpQtLib.core import resource as resource_utils

# =================================================================================

logger = None
resource = None
configs_manager = None

# =================================================================================


class tpRenamerResource(resource_utils.Resource, object):
    RESOURCES_FOLDER = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'resources')


class tpRenamer(importer.Importer, object):
    def __init__(self, *args, **kwargs):
        super(tpRenamer, self).__init__(module_name='tpRenamer', *args, **kwargs)

    def get_module_path(self):
        """
        Returns path where tpRenamer module is stored
        :return: str
        """

        try:
            mod_dir = os.path.dirname(inspect.getframeinfo(inspect.currentframe()).filename)
        except Exception:
            try:
                mod_dir = os.path.dirname(__file__)
            except Exception:
                try:
                    import tpDccLib
                    mod_dir = tpDccLib.__path__[0]
                except Exception:
                    return None

        return mod_dir

    def get_configs_path(self):
        """
        Returns path where base tpNodeGraph configurations are located
        :return: str
        """

        return os.path.join(self.get_module_path(), 'configs')

    def create_configuration_manager(self):
        """
        Creates manager that handles tpNodeGraph configuration
        :return:
        """

        from tpQtLib.core import config
        global configs_manager
        configs_manager = config.ConfigurationManager(config_paths=self.get_configs_path())


def init(do_reload=False, dev=False):
    """
    Initializes module
    :param do_reload: bool, Whether to reload modules or not
    """

    tprenamer_importer = importer.init_importer(importer_class=tpRenamer, do_reload=do_reload, debug=dev)

    global logger
    global resource
    logger = tprenamer_importer.logger
    resource = tpRenamerResource

    tprenamer_importer.import_modules()
    tprenamer_importer.import_packages(only_packages=True)
    if do_reload:
        tprenamer_importer.reload_all()

    tprenamer_importer.create_configuration_manager()


def run(do_reload=False):
    init(do_reload=do_reload)
    from tpRenamer.core import renamer
    win = renamer.run()
    return win



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Widget that manages basic rename functionality for tpRenamer
"""

from __future__ import print_function, division, absolute_import


from Qt.QtCore import *
from Qt.QtWidgets import *
from Qt.QtGui import *

from tpQtLib.core import base
from tpQtLib.widgets import splitters


class RenamerWidget(base.BaseWidget, object):

    renameUpdate = Signal()

    def __init__(self, parent=None):
        super(RenamerWidget, self).__init__(parent=parent)

    def ui(self):
        super(RenamerWidget, self).ui()

        self.main_layout.addWidget(splitters.Splitter('RENAME'))

        renamer_widget = QWidget()
        renamer_widget.setLayout(QVBoxLayout())
        renamer_widget.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Fixed)
        renamer_widget.layout().setContentsMargins(0, 0, 0, 0)
        renamer_widget.layout().setSpacing(2)
        self.main_layout.addWidget(renamer_widget)

        rename_layout = QHBoxLayout()
        rename_layout.setAlignment(Qt.AlignLeft)
        rename_layout.setContentsMargins(0, 0, 0, 0)
        rename_layout.setSpacing(2)
        renamer_widget.layout().addLayout(rename_layout)
        self._base_name_cbx = QCheckBox()
        rename_layout.addWidget(self._base_name_cbx)
        rename_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._renamer_text_lbl = QLabel('New Name: ')
        self._renamer_line = QLineEdit()
        rename_layout.addWidget(self._renamer_text_lbl)
        rename_layout.addWidget(self._renamer_line)
        reg_ex = QRegExp("^(?!^_)[a-zA-Z_]+")
        text_validator = QRegExpValidator(reg_ex, self._renamer_line)
        self._renamer_line.setValidator(text_validator)
        self._renamer_text_lbl.setEnabled(False)
        self._renamer_line.setEnabled(False)

        rename_mult_layout = QHBoxLayout()
        rename_mult_layout.setAlignment(Qt.AlignLeft)
        rename_mult_layout.setContentsMargins(0, 0, 0, 0)
        rename_mult_layout.setSpacing(5)
        renamer_widget.layout().addLayout(rename_mult_layout)
        rename_mult_layout.addItem(QSpacerItem(25, 0, QSizePolicy.Fixed, QSizePolicy.Preferred))
        rename_mult_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._rename_mult_method_lbl = QLabel('Mult. Naming: ')
        self._renamer_mult_method_combo = QComboBox()
        self._renamer_mult_method_combo.addItem('Numbers (0-9)')
        self._renamer_mult_method_combo.addItem('Letters (a-z)')
        self._renamer_mult_method_combo.setFixedWidth(100)
        rename_mult_layout.addWidget(self._rename_mult_method_lbl)
        rename_mult_layout.addWidget(self._renamer_mult_method_combo)
        self._rename_mult_method_lbl.setEnabled(False)
        self._renamer_mult_method_combo.setEnabled(False)
        self._frame_pad_lbl = QLabel('No. Padding: ')
        self._frame_pad_spin = QSpinBox()
        self._frame_pad_spin.setFocusPolicy(Qt.NoFocus)
        self._frame_pad_spin.setFixedWidth(40)
        self._frame_pad_spin.setMinimum(0)
        self._frame_pad_spin.setMaximum(10)
        self._frame_pad_lbl.setEnabled(False)
        self._frame_pad_spin.setEnabled(False)
        rename_mult_layout.addWidget(self._frame_pad_lbl)
        rename_mult_layout.addWidget(self._frame_pad_spin)
        lower_upper_grp = QButtonGroup(self)
        self._lower_radio = QRadioButton('Lower')
        self._upper_radio = QRadioButton('Upper')
        lower_upper_grp.addButton(self._lower_radio)
        lower_upper_grp.addButton(self._upper_radio)
        self._lower_radio.setVisible(False)
        self._upper_radio.setVisible(False)
        self._lower_radio.setFixedHeight(19)
        self._upper_radio.setFixedHeight(19)
        self._upper_radio.setAutoExclusive(True)
        self._lower_radio.setAutoExclusive(True)
        self._lower_radio.setChecked(True)
        self._lower_radio.setEnabled(False)
        self._upper_radio.setEnabled(False)
        rename_mult_layout.addWidget(self._lower_radio)
        rename_mult_layout.addWidget(self._upper_radio)

        renamer_widget.layout().addLayout(splitters.SplitterLayout())

        side_layout = QHBoxLayout()
        side_layout.setAlignment(Qt.AlignLeft)
        side_layout.setContentsMargins(0, 0, 0, 0)
        side_layout.setSpacing(2)
        renamer_widget.layout().addLayout(side_layout)
        self._side_cbx = QCheckBox()
        side_layout.addWidget(self._side_cbx)
        side_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._side_lbl = QLabel('Side: ')
        self._none_side = QRadioButton('None')
        self._right_side = QRadioButton('Right')
        self._center_side = QRadioButton('Center')
        self._mid_side = QRadioButton('Mid')
        self._left_side = QRadioButton('Left')
        self._none_side.setFixedHeight(15)
        self._right_side.setFixedHeight(15)
        self._center_side.setFixedHeight(15)
        self._mid_side.setFixedHeight(15)
        self._left_side.setFixedHeight(15)
        side_layout.addWidget(self._side_lbl)
        side_layout.addWidget(self._none_side)
        side_layout.addWidget(self._right_side)
        side_layout.addWidget(self._center_side)
        side_layout.addWidget(self._mid_side)
        side_layout.addWidget(self._left_side)
        self._none_side.setAutoExclusive(True)
        self._right_side.setAutoExclusive(True)
        self._center_side.setAutoExclusive(True)
        self._mid_side.setAutoExclusive(True)
        self._left_side.setAutoExclusive(True)
        self._none_side.setChecked(True)
        self._side_lbl.setEnabled(False)
        self._none_side.setEnabled(False)
        self._right_side.setEnabled(False)
        self._center_side.setEnabled(False)
        self._mid_side.setEnabled(False)
        self._left_side.setEnabled(False)

        side_layout2 = QHBoxLayout()
        side_layout2.setAlignment(Qt.AlignLeft)
        side_layout2.setContentsMargins(0, 0, 0, 0)
        side_layout2.setSpacing(2)
        renamer_widget.layout().addLayout(side_layout2)
        self._capital_side = QCheckBox('Capital?')
        side_layout2.addItem(QSpacerItem(25, 0, QSizePolicy.Fixed, QSizePolicy.Preferred))
        side_layout2.addWidget(splitters.get_horizontal_separator_widget())
        side_layout2.addWidget(self._capital_side)
        self._capital_side.setEnabled(False)

        last_joint_layout = QHBoxLayout()
        last_joint_layout.setAlignment(Qt.AlignLeft)
        last_joint_layout.setContentsMargins(0, 0, 0, 0)
        last_joint_layout.setSpacing(2)
        renamer_widget.layout().addLayout(last_joint_layout)
        self._last_joint_is_end_cbx = QCheckBox()
        self._last_joint_is_end_lbl = QLabel('Last joint is an endJoint?')
        self._last_joint_is_end_cbx.setChecked(True)
        last_joint_layout.addWidget(self._last_joint_is_end_cbx)
        last_joint_layout.addWidget(splitters.get_horizontal_separator_widget())
        last_joint_layout.addWidget(self._last_joint_is_end_lbl)

        renamer_widget.layout().addLayout(splitters.SplitterLayout())

        prefix_layout = QHBoxLayout()
        prefix_layout.setAlignment(Qt.AlignLeft)
        prefix_layout.setContentsMargins(0, 0, 0, 0)
        prefix_layout.setSpacing(2)
        renamer_widget.layout().addLayout(prefix_layout)
        self._prefix_cbx = QCheckBox()
        prefix_layout.addWidget(self._prefix_cbx)
        prefix_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._prefix_lbl = QLabel('Prefix: ')
        self._prefix_line = QLineEdit()
        self._prefix_line.setEnabled(False)
        self._prefix_line.setValidator(text_validator)
        prefix_layout.addWidget(self._prefix_lbl)
        prefix_layout.addWidget(self._prefix_line)
        self._prefix_lbl.setEnabled(False)
        self._prefix_line.setEnabled(False)

        remove_first_layout = QHBoxLayout()
        remove_first_layout.setAlignment(Qt.AlignLeft)
        remove_first_layout.setContentsMargins(0, 0, 0, 0)
        remove_first_layout.setSpacing(2)
        renamer_widget.layout().addLayout(remove_first_layout)
        self._remove_first_cbx = QCheckBox()
        remove_first_layout.addWidget(self._remove_first_cbx)
        remove_first_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._remove_first_lbl = QLabel('Remove first: ')
        self._remove_first_spn = QSpinBox()
        self._remove_first_spn.setFocusPolicy(Qt.NoFocus)
        self._remove_first_spn.setFixedWidth(40)
        self._remove_first_spn.setMinimum(0)
        self._remove_first_spn.setMaximum(99)
        last_digits_lbl = QLabel(' digits')
        remove_first_layout.addWidget(self._remove_first_lbl)
        remove_first_layout.addWidget(self._remove_first_spn)
        remove_first_layout.addWidget(last_digits_lbl)
        self._remove_first_lbl.setEnabled(False)
        self._remove_first_spn.setEnabled(False)

        renamer_widget.layout().addLayout(splitters.SplitterLayout())

        suffix_layout = QHBoxLayout()
        suffix_layout.setAlignment(Qt.AlignLeft)
        suffix_layout.setContentsMargins(0, 0, 0, 0)
        suffix_layout.setSpacing(2)
        renamer_widget.layout().addLayout(suffix_layout)
        self._suffix_cbx = QCheckBox()
        suffix_layout.addWidget(self._suffix_cbx)
        suffix_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._suffix_lbl = QLabel('Suffix: ')
        self._suffix_line = QLineEdit()
        self._suffix_line.setEnabled(False)
        self._suffix_line.setValidator(text_validator)
        suffix_layout.addWidget(self._suffix_lbl)
        suffix_layout.addWidget(self._suffix_line)
        self._suffix_lbl.setEnabled(False)
        self._suffix_line.setEnabled(False)

        remove_last_layout = QHBoxLayout()
        remove_last_layout.setAlignment(Qt.AlignLeft)
        remove_last_layout.setContentsMargins(0, 0, 0, 0)
        remove_last_layout.setSpacing(2)
        renamer_widget.layout().addLayout(remove_last_layout)
        self._remove_last_cbx = QCheckBox()
        remove_last_layout.addWidget(self._remove_last_cbx)
        remove_last_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._remove_last_lbl = QLabel('Remove last: ')
        self._remove_last_spn = QSpinBox()
        self._remove_last_spn.setFocusPolicy(Qt.NoFocus)
        self._remove_last_spn.setFixedWidth(40)
        self._remove_last_spn.setMinimum(0)
        self._remove_last_spn.setMaximum(99)
        last_digits_lbl2 = QLabel(' digits')
        remove_last_layout.addWidget(self._remove_last_lbl)
        remove_last_layout.addWidget(self._remove_last_spn)
        remove_last_layout.addWidget(last_digits_lbl2)
        self._remove_last_lbl.setEnabled(False)
        self._remove_last_spn.setEnabled(False)

    def setup_signals(self):
        self._base_name_cbx.toggled.connect(self._on_base_name_toggled)
        self._renamer_mult_method_combo.currentIndexChanged.connect(self._on_toggle_mult_naming_method)
        self._renamer_line.textChanged.connect(self._on_rename_line_text_changed)
        self._renamer_mult_method_combo.currentIndexChanged.connect(self._on_mult_method_combo_index_changed)
        self._upper_radio.clicked.connect(self._on_upper_lower_radio_clicked)
        self._lower_radio.clicked.connect(self._on_upper_lower_radio_clicked)
        self._frame_pad_spin.valueChanged.connect(self._on_padding_spinner_value_changed)

        self._side_cbx.toggled.connect(self._on_side_toggled)
        self._none_side.clicked.connect(self._on_side_radio_clcked)
        self._center_side.clicked.connect(self._on_side_radio_clcked)
        self._mid_side.clicked.connect(self._on_side_radio_clcked)
        self._left_side.clicked.connect(self._on_side_radio_clcked)
        self._right_side.clicked.connect(self._on_side_radio_clcked)
        self._capital_side.toggled.connect(self._on_side_radio_clcked)

        self._prefix_cbx.toggled.connect(self._on_prefix_toggled)
        self._prefix_line.textChanged.connect(self._on_prefx_line_text_changed)

        self._remove_first_cbx.toggled.connect(self._on_remove_first_toggled)
        self._remove_first_spn.valueChanged.connect(self._on_remove_first_value_changed)

        self._suffix_cbx.toggled.connect(self._on_suffix_toggled)
        self._suffix_line.textChanged.connect(self._on_suffix_line_text_changed)

        self._remove_last_cbx.toggled.connect(self._on_remove_last_toggled)
        self._remove_last_spn.valueChanged.connect(self._on_remove_last_value_changed)

    def get_rename_settings(self):
        """
        Internal function that returns current rename settings
        :return: str, str, str, int, bool, bool, str, bool
        """

        if self._base_name_cbx.isChecked():
            text = str(self._renamer_line.text()).strip()
        else:
            text = ''

        naming_method = bool(self._renamer_mult_method_combo.currentIndex())
        padding = 0
        upper = True

        if not naming_method:
            padding = self._frame_pad_spin.value()
        else:
            upper = self._upper_radio.isChecked()

        prefix = ''
        suffix = ''

        if self._prefix_cbx.isChecked():
            prefix = self._prefix_line.text()
        if self._suffix_cbx.isChecked():
            suffix = self._suffix_line.text()

        if not self._side_cbx.isChecked():
            side  = None
        else:
            if self._none_side.isChecked():
                side = ''
            if self._right_side.isChecked():
                if self._capital_side.isChecked():
                    side = 'R'
                else:
                    side = 'r'
            if self._center_side.isChecked():
                if self._capital_side.isChecked():
                    side = 'C'
                else:
                    side = 'c'
            if self._mid_side.isChecked():
                if self._capital_side.isChecked():
                    side = 'M'
                else:
                    side = 'm'
            if self._left_side.isChecked():
                if self._capital_side.isChecked():
                    side = 'L'
                else:
                    side = 'l'

        if not self._remove_first_cbx.isChecked():
            remove_first = 0
        else:
            remove_first = self._remove_first_spn.value()

        if not self._remove_last_cbx.isChecked():
            remove_last = 0
        else:
            remove_last = self._remove_last_spn.value()

        joint_end = self._last_joint_is_end_cbx.isChecked()

        return text, prefix, suffix, padding, naming_method, upper, side, remove_first, remove_last, joint_end

    def _on_base_name_toggled(self, flag):
        self._renamer_text_lbl.setEnabled(flag)
        self._renamer_line.setEnabled(flag)
        self._rename_mult_method_lbl.setEnabled(flag)
        self._renamer_mult_method_combo.setEnabled(flag)
        self._frame_pad_lbl.setEnabled(flag)
        self._frame_pad_spin.setEnabled(flag)
        self._lower_radio.setEnabled(flag)
        self._upper_radio.setEnabled(flag)
        self.renameUpdate.emit()

    def _on_side_toggled(self, flag):
        self._side_lbl.setEnabled(flag)
        self._none_side.setEnabled(flag)
        self._center_side.setEnabled(flag)
        self._left_side.setEnabled(flag)
        self._right_side.setEnabled(flag)
        self._mid_side.setEnabled(flag)
        self._capital_side.setEnabled(flag)
        self.renameUpdate.emit()

    def _on_prefix_toggled(self, flag):
        self._prefix_lbl.setEnabled(flag)
        self._prefix_line.setEnabled(flag)
        self.renameUpdate.emit()

    def _on_remove_first_toggled(self, flag):
        self._remove_first_lbl.setEnabled(flag)
        self._remove_first_spn.setEnabled(flag)
        self.renameUpdate.emit()

    def _on_suffix_toggled(self, flag):
        self._suffix_lbl.setEnabled(flag)
        self._suffix_line.setEnabled(flag)
        self.renameUpdate.emit()

    def _on_remove_last_toggled(self, flag):
        self._remove_last_lbl.setEnabled(flag)
        self._remove_last_spn.setEnabled(flag)
        self.renameUpdate.emit()

    def _on_toggle_mult_naming_method(self, index):

        """
        Method that updates the status of the radio buttons considering which option es enabled
        """

        self._lower_radio.setVisible(index)
        self._upper_radio.setVisible(index)
        self._frame_pad_lbl.setVisible(not index)
        self._frame_pad_spin.setVisible(not index)

    def _on_rename_line_text_changed(self, new_text):
        self.renameUpdate.emit()

    def _on_mult_method_combo_index_changed(self, index):
        self.renameUpdate.emit()

    def _on_upper_lower_radio_clicked(self):
        self.renameUpdate.emit()

    def _on_padding_spinner_value_changed(self, value):
        self.renameUpdate.emit()

    def _on_side_radio_clcked(self):
        self.renameUpdate.emit()

    def _on_prefx_line_text_changed(self, new_text):
        self.renameUpdate.emit()

    def _on_suffix_line_text_changed(self, new_text):
        self.renameUpdate.emit()

    def _on_remove_first_value_changed(self, value):
        self.renameUpdate.emit()

    def _on_remove_last_value_changed(self, value):
        self.renameUpdate.emit()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Widget that contains manual rename widgets for tpRenamer
"""

from __future__ import print_function, division, absolute_import

from Qt.QtCore import *

from tpQtLib.core import base

from tpRenamer.widgets import renamerwidget, replacerwidget


class ManualRenameWidget(base.BaseWidget, object):

    renameUpdate = Signal()
    replaceUpdate = Signal()

    def __init__(self, parent=None):
        super(ManualRenameWidget, self).__init__(parent=parent)

    def ui(self):
        super(ManualRenameWidget, self).ui()

        self._renamer_widget = renamerwidget.RenamerWidget()
        self._replacer_widget = replacerwidget.ReplacerWidget()

        self.main_layout.addWidget(self._renamer_widget)
        self.main_layout.addWidget(self._replacer_widget)

        # extra_layout = QHBoxLayout()
        # extra_layout.setContentsMargins(10, 10, 10, 10)
        # self.main_layout.addLayout(extra_layout)
        #
        # self._select_hierarchy_btn = QPushButton('Select Hierarchy')
        # extra_layout.addWidget(self._select_hierarchy_btn)

    def setup_signals(self):
        self._renamer_widget.renameUpdate.connect(self.renameUpdate.emit)
        self._replacer_widget.replaceUpdate.connect(self.replaceUpdate.emit)

    def get_rename_settings(self):
        """
        Function that returns current rename settings
        :return: str, str, str, int, bool, bool, str, bool
        """

        return self._renamer_widget.get_rename_settings()

    def get_replace_settings(self):
        """
        Function that returns current replace settings
        :return: str, str, str, int, bool, bool, str, bool
        """

        return self._replacer_widget.get_replace_settings()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Fully featured renamer tool
"""

from __future__ import print_function, division, absolute_import

import traceback
from functools import partial

from Qt.QtCore import *
from Qt.QtWidgets import *

import tpDccLib as tp
from tpPyUtils import decorators, strings
from tpQtLib.core import base, window
from tpQtLib.widgets import splitters, stack

if tp.is_maya():
    import tpMayaLib as maya
    from tpMayaLib.core import decorators as maya_decorators
    undo_decorator = maya_decorators.undo
else:
    undo_decorator = decorators.empty_decorator

from tpNameIt.core import namelib

import tpRenamer


class Renamer(window.MainWindow, object):
    def __init__(self):
        super(Renamer, self).__init__(
            name='RenamerBaseWindow',
            title='Renamer',
            size=(350, 700),
            fixed_size=False,
            auto_run=True,
            frame_less=True,
            use_style=False
        )

        self.setWindowIcon(tpRenamer.resource.icon('rename'))

    def ui(self):
        super(Renamer, self).ui()

        self._renamer_widget = RenamerWidget()
        self.main_layout.addWidget(self._renamer_widget)


class RenamerWidget(base.BaseWidget, object):

    NAMING_LIB = namelib.NameLib

    def __init__(self, config=None, parent=None):
        self._name_lib = self.NAMING_LIB()
        self._config = config or tpRenamer.configs_manager.get_config('renamer')
        super(RenamerWidget, self).__init__(parent=parent)

    def ui(self):
        super(RenamerWidget, self).ui()

        from tpRenamer.widgets import manualrenamewidget, autorenamewidget

        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        self._top_layout = QHBoxLayout()
        self._top_layout.setAlignment(Qt.AlignLeft)
        self._top_layout.setContentsMargins(2, 2, 2, 2)
        self._top_layout.setSpacing(2)
        self._buttons_grp = QButtonGroup(self)
        self._buttons_grp.setExclusive(True)
        self.main_layout.addLayout(self._top_layout)
        self.main_layout.addLayout(splitters.SplitterLayout())

        self._categories_layout = QHBoxLayout()
        self._categories_layout.setAlignment(Qt.AlignLeft)
        self._categories_layout.setContentsMargins(2, 2, 2, 2)
        self._categories_layout.setSpacing(2)
        self._top_layout.addLayout(self._categories_layout)

        self._top_layout.addWidget(splitters.get_horizontal_separator_widget())
        selection_layout = QHBoxLayout()
        selection_layout.setContentsMargins(4, 0, 4, 0)
        selection_layout.setSpacing(2)
        self._top_layout.addLayout(selection_layout)

        selection_mode_lbl = QLabel('Selection: ')
        self._all_radio = QRadioButton('All')
        self._all_radio.setFixedHeight(19)
        self._all_radio.setChecked(True)
        self._all_radio.setAutoExclusive(True)
        self._selected_radio = QRadioButton('Selected')
        self._selected_radio.setFixedHeight(19)
        self._selected_radio.setAutoExclusive(True)
        self._hierarchy_cbx = QCheckBox('Hierarchy')
        self._hierarchy_cbx.setFixedHeight(19)

        selection_layout.addWidget(selection_mode_lbl)
        selection_layout.addWidget(self._all_radio)
        selection_layout.addWidget(self._selected_radio)
        selection_layout.addWidget(self._hierarchy_cbx)

        self._splitter = QSplitter(Qt.Horizontal)
        self._splitter.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        self.main_layout.addWidget(self._splitter)

        self.rename_tab = QTabWidget()
        self._splitter.addWidget(self.rename_tab)

        self.manual_rename_widget = manualrenamewidget.ManualRenameWidget()
        self.auto_rename_widget = autorenamewidget.AutoRenameWidget(naming_lib=self._name_lib)

        self.rename_tab.addTab(self.manual_rename_widget, 'Manual')
        self.rename_tab.addTab(self.auto_rename_widget, 'Auto')

        self._stack = stack.SlidingStackedWidget()
        self._splitter.addWidget(self._stack)

        no_items_widget = QFrame()
        no_items_widget.setFrameShape(QFrame.StyledPanel)
        no_items_widget.setFrameShadow(QFrame.Sunken)
        no_items_widget.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        no_items_layout = QVBoxLayout()
        no_items_layout.setContentsMargins(0, 0, 0, 0)
        no_items_layout.setSpacing(0)
        no_items_widget.setLayout(no_items_layout)
        no_items_lbl = QLabel()
        no_items_pixmap = tpRenamer.resource.pixmap('no_items')
        no_items_lbl.setPixmap(no_items_pixmap)
        no_items_lbl.setAlignment(Qt.AlignCenter)
        no_items_layout.addItem(QSpacerItem(0, 10, QSizePolicy.Preferred, QSizePolicy.Expanding))
        no_items_layout.addWidget(no_items_lbl)
        no_items_layout.addItem(QSpacerItem(0, 10, QSizePolicy.Preferred, QSizePolicy.Expanding))

        self._stack.addWidget(no_items_widget)

        self._setup_categories()

    def setup_signals(self):
        self._stack.animFinished.connect(self._on_stack_anim_finished)
        self.rename_tab.currentChanged.connect(self._on_tab_changed)
        self._all_radio.clicked.connect(self._on_refresh_category)
        self._selected_radio.clicked.connect(self._on_refresh_category)
        self._hierarchy_cbx.toggled.connect(self._on_toggle_hierarchy_cbx)
        self.manual_rename_widget.renameUpdate.connect(self.update_current_items)
        self.manual_rename_widget.replaceUpdate.connect(self.update_current_items)
        self.auto_rename_widget.renameUpdated.connect(self.update_current_items)

    def keyPressEvent(self, event):

        # Prevent lost focus when writing on QLineEdits
        if event.key() in (Qt.Key_Shift, Qt.Key_Control, Qt.Key_CapsLock):
            event.accept()
        else:
            event.ignore()

    def update_current_items(self):
        """
        Function that updates the names of the current selected items
        """

        self._update_current_objects_items()

    def _setup_categories(self):

        from tpRenamer.widgets import categorywidget

        if not self._config:
            tpRenamer.logger.warning(
                'Impossible to setup categories because tpRenamer configuration file is not available!')
            return

        categories = self._config.get('categories', default=list())
        if not categories:
            tpRenamer.logger.warning(
                'Impossible to setup categories because categories property is not defined in tpRenamer '
                'configuration file!')
            return

        for i, category in enumerate(categories):
            for category_name, category_data in category.items():
                title = category_data.get('title', category)
                icon = category_data.get('icon', None)
                types = category_data.get('types', dict())
                nodes_to_discard = self._get_nodes_to_discard()

                category_btn = QPushButton(title)
                category_btn.setMinimumWidth(55)
                category_btn.setCheckable(True)
                if icon:
                    category_btn.setIcon(tpRenamer.resource.icon(icon))
                if i == 0:
                    category_btn.setChecked(True)
                self._buttons_grp.addButton(category_btn)
                self._categories_layout.addWidget(category_btn)
                category_widget = categorywidget.CategoryWidget(types=types, nodes_to_discard=nodes_to_discard)
                self._stack.addWidget(category_widget)

                category_widget.doRefresh.connect(self._on_refresh_category)
                category_widget.doPreview.connect(self._set_preview_names)
                category_widget.togglePreview.connect(self.update_current_items)
                category_widget.doRename.connect(self._on_rename)
                category_btn.clicked.connect(partial(self._on_category_selected, i+1))

        if self._stack.count() > 1:
            self._on_category_selected(1)

    def _get_nodes_to_discard(self):
        """
        Internal function that updates name list with current scene objects
        """

        discard_nodes = self._config.get('nodes_to_discard', default=list())
        types_to_discard = self._config.get('types_to_discard', default=list())
        for node_type in types_to_discard:
            discard_nodes.extend(tp.Dcc.list_nodes(node_type=node_type))

        return discard_nodes

    def _update_current_objects_items(self):
        category_widget = self._stack.current_widget
        if not category_widget:
            return

        names_list = category_widget.get_names_list()
        for i in range(names_list.topLevelItemCount()):
            item = names_list.topLevelItem(i)
            item.preview_name = ''
            if hasattr(item, 'obj'):
                item.setText(0, item.obj)

        if category_widget.is_preview_enabled():
            selected_items = names_list.selectedItems()
            self._set_preview_names(items=selected_items)

    def _find_manual_available_name(self, items, name, prefix=None, suffix=None, side='', index=-1, padding=0, letters=False, capital=False, remove_first=0, remove_last=0, find_str=None, replace_str=None, joint_end=False):
        """
        Recursively find a free name matching specified criteria
        @param name: str, Name to check if already exists in the scene
        @param suffix: str, Suffix for the name
        @param index: int, Index of the name
        @param padding: int, Padding for the characters/numbers
        @param letters: bool, True if we want to use letters when renaming multiple nodes
        @param capital: bool, True if we want letters to be capital
        """

        if prefix:
            if side and side != '':
                test_name = '{}_{}_{}'.format(prefix, side, name)
            else:
                test_name = '{}_{}'.format(prefix, name)
        else:
            if side and side != '':
                test_name = '{}_{}'.format(side, name)
            else:
                test_name = name

        if index >= 0:
            if letters is True:
                letter = strings.get_alpha(index, capital)
                test_name = '{}_{}'.format(test_name, letter)
            else:
                test_name = '{}_{}'.format(test_name, str(index).zfill(padding+1))

        if suffix:
            test_name = '{}_{}'.format(test_name, suffix)

        if remove_first and remove_first > 0:
            test_name = test_name[remove_first:]

        if remove_last and remove_last > 0:
            test_name = test_name[:-remove_last]

        if find_str != None and find_str != '' and replace_str != None:
            test_name = test_name.replace(find_str, replace_str)

        item_names = [item.obj for item in items]

        # if object exists, try next index
        if tp.Dcc.object_exists(test_name) or test_name in item_names:
            new_index = int(index) + 1
            return self._find_manual_available_name(
                items, name, prefix=prefix, index=new_index, padding=padding,
                letters=letters, capital=capital, remove_first=remove_first, remove_last=remove_last,
                joint_end=joint_end, find_str=find_str, replace_str=replace_str
            )

        return test_name

    def _generate_preview_names(self, items):

        # Manual Rename
        if self.rename_tab.currentIndex() == 0:

            text, prefix, suffix, padding, naming_method, upper, side, remove_first, remove_last, joint_end = self.manual_rename_widget.get_rename_settings()
            find_str, replace_str = self.manual_rename_widget.get_replace_settings()

            duplicated_names = dict()
            generated_names = list()

            for item in items:
                if not text:
                    base_name = item.obj
                else:
                    base_name = text

                if base_name == item.obj and not prefix and not suffix and not side:
                    generate_preview_name = False
                else:
                    generate_preview_name = True
                if base_name in duplicated_names:
                    duplicated_names[base_name] += 1
                else:
                    duplicated_names[base_name] = 0

                if generate_preview_name:

                    if base_name == item.obj and (prefix or suffix or side):
                        index = None
                    else:
                        index = duplicated_names[base_name]

                    preview_name = self._find_manual_available_name(items, base_name, prefix=prefix, side=side, suffix=suffix,
                                                                    index=index, padding=padding,
                                                                    letters=naming_method, capital=upper, joint_end=joint_end,
                                                                    remove_first=remove_first, remove_last=remove_last,
                                                                    find_str=find_str, replace_str=replace_str)

                    while preview_name in generated_names:
                        duplicated_names[base_name] += 1
                        preview_name = self._find_manual_available_name(items, base_name, prefix=prefix, side=side, suffix=suffix,
                                                                        index=duplicated_names[base_name], padding=padding,
                                                                        letters=naming_method, capital=upper, joint_end=joint_end,
                                                                        remove_first=remove_first, remove_last=remove_last,
                                                                        find_str=find_str, replace_str=replace_str)
                else:
                    preview_name = base_name

                item.preview_name = preview_name
                generated_names.append(preview_name)
        else:
            data = self.auto_rename_widget.get_rename_settings()

            for i, item in enumerate(items):
                data['id'] = i
                preview_name = self._name_lib.solve(**data)
                item.preview_name = preview_name

    def _set_preview_names(self, items, do_preview=True):
        """
        Internal function that sets the preview name for the given items
        :param items:
        :return:
        """

        self._generate_preview_names(items)

        if do_preview:
            for item in items:
                if item.preview_name:
                    item.setText(0, item.preview_name)

    def _on_tab_changed(self, tab_index):

        if tab_index != 1:
            return

        self.auto_rename_widget.update_rules()

    def _on_toggle_hierarchy_cbx(self, flag):
        self._on_refresh_category()

    def _on_category_selected(self, index):
        if self._stack.currentIndex() == index:
            return

        for i, btn in enumerate(self._buttons_grp.buttons()):
            if i == index - 1:
                continue
            btn.setEnabled(False)
        self._stack.slide_in_index(index)

    def _on_stack_anim_finished(self, index):
        for btn in self._buttons_grp.buttons():
            btn.setEnabled(True)
        category_widget = self._stack.current_widget
        if not category_widget:
            return
        self._on_refresh_category(category_widget)

    def _on_refresh_category(self, category_widget=None):
        if not category_widget:
            category_widget = self._stack.current_widget
            if not category_widget:
                return

        category_widget.refresh(
            selected_objects=self._selected_radio.isChecked(), hierarchy=self._hierarchy_cbx.isChecked())

    @undo_decorator
    def _on_rename(self):
        category_widget = self._stack.current_widget
        if not category_widget:
            return

        names_list = category_widget.get_names_list()
        items_to_rename = names_list.selectedItems()
        for item in items_to_rename:
            try:
                if hasattr(item, 'handle'):
                    if tp.is_maya():
                        mobj = item.handle.object()
                        try:
                            dag_path = maya.OpenMaya.MDagPath.getAPathTo(mobj)
                            full_name = dag_path.fullPathName()
                        except Exception:
                            full_name = item.full_name
                        try:
                            tp.Dcc.rename_node(full_name, item.preview_name)
                            item.obj = item.preview_name
                            item.preview_name = ''
                        except Exception as exc:
                            tpRenamer.logger.warning('Impossible to rename: {} > {} | {}'.format(
                                full_name, item.preview_name, exc
                            ))
                else:
                    tp.Dcc.rename_node(item.full_name, item.preview_name)
                    item.obj = item.preview_name
                    item.preview_name = ''
            except Exception as e:
                tpRenamer.logger.warning('Impossible to rename: {} to {}'.format(item.full_name, item.preview_name))
                tpRenamer.logger.error('{} | {}'.format(e, traceback.format_exc()))

        self.update_current_items()
        names_list.clearSelection()


def run():
    win = Renamer()
    win.show()
    return win



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Widget that manages replace functionality for tpRenamer
"""

from __future__ import print_function, division, absolute_import


from Qt.QtCore import *
from Qt.QtWidgets import *
from Qt.QtGui import *

from tpQtLib.core import base
from tpQtLib.widgets import splitters


class ReplacerWidget(base.BaseWidget, object):

    replaceUpdate = Signal()

    def __init__(self, parent=None):
        super(ReplacerWidget, self).__init__(parent=parent)

    def ui(self):
        super(ReplacerWidget, self).ui()

        self.main_layout.addWidget(splitters.Splitter('REPLACE'))

        replace_layout = QHBoxLayout()
        replace_layout.setAlignment(Qt.AlignLeft)
        replace_layout.setContentsMargins(0, 0, 0, 0)
        replace_layout.setSpacing(2)
        self.main_layout.addLayout(replace_layout)

        self._find_replace_cbx = QCheckBox()
        replace_layout.addWidget(self._find_replace_cbx)
        replace_layout.addWidget(splitters.get_horizontal_separator_widget())

        self._replace_lbl = QLabel('Find: ')
        self._replace_line = QLineEdit()
        self._with_lbl = QLabel('Replace: ')
        self._with_line = QLineEdit()
        # replace_lbl.setFixedWidth(55)
        # with_lbl.setFixedWidth(55)
        reg_ex = QRegExp("[a-zA-Z_0-9]+")
        text_validator = QRegExpValidator(reg_ex, self._replace_line)
        self._replace_line.setValidator(text_validator)
        self._with_line.setValidator(text_validator)

        replace_layout.addWidget(self._replace_lbl)
        replace_layout.addWidget(self._replace_line)
        replace_layout.addItem(QSpacerItem(10, 0, QSizePolicy.Fixed, QSizePolicy.Preferred))
        replace_layout.addWidget(self._with_lbl)
        replace_layout.addWidget(self._with_line)

        self._replace_lbl.setEnabled(False)
        self._replace_line.setEnabled(False)
        self._with_lbl.setEnabled(False)
        self._with_line.setEnabled(False)

    def setup_signals(self):
        self._find_replace_cbx.toggled.connect(self._on_find_replace_toggled)
        self._replace_line.textChanged.connect(self._on_replace_line_text_changed)
        self._with_line.textChanged.connect(self._on_with_line_text_changed)

    def get_replace_settings(self):
        """
        Internal function that returns current rename settings
        :return: str, str
        """

        if self._find_replace_cbx.isChecked():
            find_str = self._replace_line.text()
            replace_str = self._with_line.text()
        else:
            find_str = ''
            replace_str = ''

        return find_str, replace_str

    def _on_find_replace_toggled(self, flag):

        self._replace_lbl.setEnabled(flag)
        self._replace_line.setEnabled(flag)
        self._with_lbl.setEnabled(flag)
        self._with_line.setEnabled(flag)
        self.replaceUpdate.emit()

    def _on_replace_line_text_changed(self, new_text):
        self.replaceUpdate.emit()

    def _on_with_line_text_changed(self, new_text):
        self.replaceUpdate.emit()


#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Widget that contains auto rename widgets for tpRenamer
"""

from __future__ import print_function, division, absolute_import

import traceback
from functools import partial
from collections import OrderedDict

from Qt.QtCore import *
from Qt.QtWidgets import *

from tpQtLib.core import base, qtutils
from tpQtLib.widgets import buttons

import tpRenamer


class AutoRenameWidget(base.BaseWidget, object):

    renameUpdated = Signal()

    def __init__(self, naming_lib, parent=None):
        self._naming_lib = naming_lib
        super(AutoRenameWidget, self).__init__(parent=parent)

        self._token_widgets = dict()

    def ui(self):
        super(AutoRenameWidget, self).ui()

        main_splitter = QSplitter(Qt.Horizontal)
        main_splitter.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        self.main_layout.addWidget(main_splitter)

        auto_widget = QWidget()
        auto_layout = QVBoxLayout()
        auto_widget.setLayout(auto_layout)
        main_splitter.addWidget(auto_widget)

        edit_icon = tpRenamer.resource.icon('edit', extension='png')
        self.edit_btn = buttons.IconButton(icon=edit_icon, icon_padding=2, button_style=buttons.ButtonStyles.FlatStyle)
        self.rules_list = QTreeWidget(self)
        self.rules_list.setHeaderHidden(True)
        self.rules_list.setSortingEnabled(True)
        self.rules_list.setRootIsDecorated(False)
        self.rules_list.setSelectionMode(QAbstractItemView.SingleSelection)
        self.rules_list.sortByColumn(0, Qt.AscendingOrder)
        self.rules_list.setUniformRowHeights(True)
        self.rules_list.setAlternatingRowColors(True)
        self.rules_list.setStyleSheet(
            '''
            QTreeView{alternate-background-color: #3b3b3b;}
            QTreeView::item {padding:3px;}
            QTreeView::item:!selected:hover {
                background-color: #5b5b5b;
                margin-left:-3px;
                border-left:0px;
            }
            QTreeView::item:selected {
                background-color: #48546a;
                border-left:2px solid #6f93cf;
                padding-left:2px;
            }
            QTreeView::item:selected:hover {
                background-color: #586c7a;
                border-left:2px solid #6f93cf;
                padding-left:2px;
            }
            '''
        )

        auto_layout.addWidget(self.edit_btn)
        auto_layout.addWidget(self.rules_list)

        auto_w = QWidget()
        self.auto_l = QVBoxLayout()
        auto_w.setLayout(self.auto_l)
        auto_w.setMinimumWidth(200)
        main_splitter.addWidget(auto_w)

        self.main_auto_layout = QFormLayout()
        self.auto_l.addLayout(self.main_auto_layout)

    def setup_signals(self):
        self.edit_btn.clicked.connect(self._on_open_naming_manager)
        self.rules_list.itemSelectionChanged.connect(self._on_item_selection_changed)

    def get_rename_settings(self):
        rename_settings = dict()

        if not self._token_widgets:
            return rename_settings

        for token_name, widget in self._token_widgets.items():
            if isinstance(widget, list):
                rename_settings[token_name] = widget[0].currentText()
            elif isinstance(widget, QLineEdit):
                rename_settings[token_name] = widget.text()

        return rename_settings

    def add_token(self, token_name, line_layout):
        self.main_auto_layout.addRow(token_name, line_layout)

    def update_rules(self):
        self.rules_list.blockSignals(True)

        try:
            self.rules_list.clear()
            item_to_select = None
            current_rule = self._naming_lib.active_rule()
            qtutils.clear_layout(self.main_auto_layout)
            rules = self._naming_lib.rules
            for rule in rules:
                item = QTreeWidgetItem(self.rules_list, [rule.name])
                item.rule = rule
                self.rules_list.addTopLevelItem(item)
                self.rules_list.setCurrentItem(item)
                if current_rule and current_rule.name() == rule.name:
                    item_to_select = item
                self.rules_list.setItemSelected(item, False)
            if item_to_select:
                self.rules_list.setItemSelected(item_to_select, True)
        except Exception as e:
            tpRenamer.logger.error('{} | {}'.format(e, traceback.format_exc()))
        finally:
            self.rules_list.blockSignals(False)

        self._on_change_name_rule()

    def _on_change_name_rule(self):

        qtutils.clear_layout(self.main_auto_layout)

        rule_item = self.rules_list.currentItem()
        if not rule_item:
            return

        current_rule = self._naming_lib.active_rule()
        self._naming_lib.set_active_rule(name=rule_item.rule.name)
        rule_tokens = self._naming_lib.tokens

        active_tokens = list()
        active_rule = self._naming_lib.active_rule()
        for field in active_rule.fields():
            for token in rule_tokens:
                if token.name == field:
                    active_tokens.append(token)

        self._token_widgets = OrderedDict()

        for token in reversed(active_tokens):
            token_name = token.name
            token_value = token.get_items()
            token_default = token.default

            if token_name == 'id':
                continue

            if token_value:
                w = QComboBox()
                w_l = QLabel()
                w.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
                w_l.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
                self._token_widgets[token_name] = [w, w_l]
                self.add_token(token_name, qtutils.get_line_layout('', self, w, QLabel(u'\u25ba'), w_l))
                for key, value in token_value.items():
                    if key == 'default':
                        continue
                    w.addItem(key)
                try:
                    if token_default > 0:
                        w.setCurrentIndex(token_default-1)
                except Exception:
                    w.setCurrentIndex(0)
                current_text = w.currentText()
                try:
                    current_value = token.solve(self._naming_lib.active_rule(), current_text)
                    w_l.setText(str(current_value))
                except Exception:
                    pass
                w.currentTextChanged.connect(partial(self._on_combo_changed, token))
            else:
                w = QLineEdit(self)
                w.textChanged.connect(self._on_text_changed)
                w.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
                self._token_widgets[token_name] = w
                self.add_token(token_name, qtutils.get_line_layout('', self, w))

        if current_rule:
            self._naming_lib.set_active_rule(current_rule.name)

    def _on_combo_changed(self, token, text):
        token_name = token.name
        if token_name not in self._token_widgets:
            return
        token_widgets = self._token_widgets[token_name]
        try:
            current_value = token.solve(self._naming_lib.active_rule(), text)
            token_widgets[1].setText(str(current_value))
        except Exception:
            pass
        self.renameUpdated.emit()

    def _on_text_changed(self, text):
        self.renameUpdated.emit()

    def _on_open_naming_manager(self):
        from tpNameIt.core import nameit
        win = nameit.run()
        return win

    def _on_item_selection_changed(self):
        current_item = self.rules_list.currentItem()
        current_rule = current_item.rule
        self._naming_lib.set_active_rule(current_rule.name)
        self._on_change_name_rule()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains exceptions used by tpRenamer
"""

from __future__ import print_function, division, absolute_import

import tpDccLib as tp


class RenameException(Exception):
    """
    Custom exception class that will handle errors when renaming elements using tpRenamer tool
    """

    def __init__(self, nodes):

        error_text = '======= Renamer: Failed to rename one or more nodes ======='
        if not hasattr(nodes, '__iter__'):
            nodes = [nodes]
        for node in nodes:
            if not tp.Dcc.object_exists(node):
                error_text += "\t'%s' no longer exists.\n" % node
            elif tp.Dcc.node_is_locked(node):
                error_text += "\t'%s' is locked.\n" % node
            else:
                error_text += "\t'%s' failure unknows.\n" % node

        Exception.__init__(self, error_text)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains category widget implementation
"""

from __future__ import print_function, division, absolute_import

from Qt.QtCore import *
from Qt.QtWidgets import *

import tpQtLib
import tpDccLib as tp
from tpQtLib.core import base
from tpQtLib.widgets import search, buttons, splitters

if tp.is_maya():
    import tpMayaLib as maya


class CategoryWidget(base.BaseWidget, object):

    doRefresh = Signal(object)
    doPreview = Signal(list, bool)
    doRename = Signal()
    togglePreview = Signal()

    def __init__(self, types, nodes_to_discard=None, parent=None):
        self._types = types
        self._default_nodes_to_discard = nodes_to_discard or list()
        self._category_buttons = list()
        super(CategoryWidget, self).__init__(parent=parent)

    @property
    def types(self):
        return self._types

    @property
    def nodes_to_discard(self):
        return self._default_nodes_to_discard

    @nodes_to_discard.setter
    def nodes_to_discard(self, nodes_to_discard):
        self._default_nodes_to_discard = nodes_to_discard

    def ui(self):
        super(CategoryWidget, self).ui()

        filter_layout = QHBoxLayout()
        filter_layout.setContentsMargins(10, 0, 10, 0)
        filter_layout.setSpacing(2)
        self.main_layout.addLayout(filter_layout)
        refresh_icon = tpQtLib.resource.icon('refresh')
        self._refresh_list_btn = buttons.IconButton(
            icon=refresh_icon, icon_padding=2, button_style=buttons.ButtonStyles.FlatStyle)
        self._names_filter = search.SearchFindWidget()
        self._search_lbl = QLabel('0 found')
        filter_layout.addWidget(self._refresh_list_btn)
        filter_layout.addWidget(self._names_filter)
        filter_layout.addWidget(self._search_lbl)

        self._types_layout = QHBoxLayout()
        self._types_layout.setContentsMargins(0, 0, 0, 0)
        self._types_layout.setSpacing(2)
        self.main_layout.addLayout(self._types_layout)

        self._names_list = QTreeWidget(self)
        self._names_list.setHeaderHidden(True)
        self._names_list.setSortingEnabled(False)
        self._names_list.setRootIsDecorated(False)
        self._names_list.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self._names_list.sortByColumn(0, Qt.AscendingOrder)
        self._names_list.setUniformRowHeights(True)
        self._names_list.setAlternatingRowColors(True)

        self.main_layout.addWidget(self._names_list)

        bottom_buttons_layout = QHBoxLayout()
        bottom_buttons_layout.setAlignment(Qt.AlignLeft)
        bottom_buttons_layout.setContentsMargins(2, 2, 2, 2)
        bottom_buttons_layout.setSpacing(2)
        self.main_layout.addLayout(bottom_buttons_layout)

        preview_icon = tpQtLib.resource.icon('preview')
        self._sort_btn = QPushButton('Sort')
        self._sort_btn.setMinimumWidth(40)
        self._all_btn = QPushButton('All')
        self._all_btn.setMinimumWidth(40)
        self._none_btn = QPushButton('None')
        self._none_btn.setMinimumWidth(40)
        bottom_buttons_layout.addWidget(self._sort_btn)
        bottom_buttons_layout.addWidget(self._all_btn)
        bottom_buttons_layout.addWidget(self._none_btn)

        bottom_buttons_layout.addWidget(splitters.get_horizontal_separator_widget())
        self._hide_default_scene_nodes_cbx = QCheckBox('Hide Default Scene Objects')
        self._hide_default_scene_nodes_cbx.setChecked(True)
        bottom_buttons_layout.addWidget(self._hide_default_scene_nodes_cbx)

        self.main_layout.addLayout(splitters.SplitterLayout())

        preview_layout = QHBoxLayout()
        preview_layout.setContentsMargins(0, 0, 0, 0)
        preview_layout.setSpacing(2)
        self.main_layout.addLayout(preview_layout)

        self._preview_btn = QPushButton('Preview')
        self._preview_btn.setIcon(preview_icon)
        self._preview_btn.setCheckable(True)
        self._preview_btn.setChecked(True)
        self._preview_btn.setMinimumWidth(100)
        self._preview_btn.setMaximumWidth(100)
        self._rename_btn = QPushButton('Select objects in the list to rename ...')
        self._rename_btn.setEnabled(False)
        preview_layout.addWidget(self._preview_btn)
        preview_layout.addWidget(self._rename_btn)

        self._setup_types()
        self.refresh()

    def setup_signals(self):
        self._refresh_list_btn.clicked.connect(self._on_refresh_list)
        self._names_filter.textChanged.connect(self._on_filter_names_changed)
        self._hide_default_scene_nodes_cbx.toggled.connect(self._on_toggle_hide_default_scene_nodes_cbx)
        self._names_list.itemSelectionChanged.connect(self._on_item_selection_changed)
        self._none_btn.clicked.connect(self._on_select_none_clicked)
        self._all_btn.clicked.connect(self._on_select_all_clicked)
        self._preview_btn.toggled.connect(self._on_toggle_preview)
        self._rename_btn.clicked.connect(self.doRename.emit)

    def get_names_list(self):
        return self._names_list

    def is_preview_enabled(self):
        return self._preview_btn.isChecked()

    def refresh(self, selected_objects=False, hierarchy=False):
        self._names_list.clear()
        self._names_list.setSortingEnabled(True)

        try:
            objs_names = list()
            if not selected_objects:
                objs_names.extend(tp.Dcc.all_scene_objects(full_path=True))
            else:
                objs_names.extend(tp.Dcc.selected_nodes(full_path=True))
                if objs_names and hierarchy:
                    children_list = list()
                    for obj in objs_names:
                        children = tp.Dcc.list_children(obj, all_hierarchy=True, full_path=True)
                        if children:
                            children_list.extend(children)
                    children_list = list(set(children_list))
                    objs_names.extend(children_list)
            self._update_names_list(objs_names)
            self._on_filter_names_changed(self._names_filter.get_text())
        finally:
            self._names_list.setSortingEnabled(False)

    def _setup_types(self):
        for i, category_type in enumerate(self._types):
            for type_name, type_data in category_type.items():
                dcc_type = type_data.get('type', None)
                dcc_fn = type_data.get('fn', None)
                dcc_args = type_data.get('args', dict())
                type_btn = QPushButton(type_name)
                type_btn.setCheckable(True)
                type_btn.setProperty('dcc_type', dcc_type)
                type_btn.setProperty('dcc_fn', dcc_fn)
                type_btn.setProperty('dcc_args', dcc_args)
                if i == 0:
                    type_btn.setChecked(True)
                self._types_layout.addWidget(type_btn)
                self._category_buttons.append(type_btn)
                type_btn.toggled.connect(self._on_toggle_type)

        self._others_btn = QPushButton('Others')
        self._others_btn.setCheckable(True)
        self._types_layout.addWidget(self._others_btn)
        self._others_btn.toggled.connect(self._on_toggle_type)

    def _get_nodes_to_discard(self):
        """
        Internal function that returns list of nodes that should be discarded during renaming process
        """

        discard_nodes = self._default_nodes_to_discard[:] or list()

        if self._hide_default_scene_nodes_cbx and self._hide_default_scene_nodes_cbx.isChecked():
            discard_nodes.extend(tp.Dcc.default_scene_nodes(full_path=False))

        # discard_nodes.extend(tp.Dcc.list_nodes(node_type='camera'))

        for btn in self._category_buttons:
            if not btn.isChecked():
                dcc_type = btn.property('dcc_type')
                if dcc_type:
                    discard_nodes.extend(tp.Dcc.list_nodes(node_type=btn.property('dcc_type')))
                else:
                    dcc_fn = btn.property('dcc_fn')
                    if dcc_fn:
                        dcc_args = btn.property('dcc_args')
                        if tp.is_maya():
                            valid_args = dict()
                            for arg_name, arg_value in dcc_args.items():
                                valid_args[str(arg_name)] = arg_value
                            nodes = getattr(maya.cmds, dcc_fn)(**valid_args)
                            discard_nodes.extend(nodes)

        return list(set(discard_nodes))

    def _get_node_types(self):
        node_types = set()
        for btn in self._category_buttons:
            dcc_type = btn.property('dcc_type')
            if dcc_type:
                node_types.add(dcc_type)
            else:
                dcc_fn = btn.property('dcc_fn')
                if dcc_fn:
                    dcc_args = btn.property('dcc_args')
                    if tp.is_maya():
                        valid_args = dict()
                        for arg_name, arg_value in dcc_args.items():
                            valid_args[str(arg_name)] = arg_value
                        nodes = getattr(maya.cmds, dcc_fn)(**valid_args)
                        for node in nodes:
                            node_type = tp.Dcc.node_type(node)
                            node_types.add(node_type)

        return list(node_types)

    def _update_names_list(self, nodes):
        """
        Internal function that updates names list with given node names
        :param nodes: list(str)
        """

        nodes_to_discard = self._get_nodes_to_discard() or list()

        nodes = list(set(nodes))

        for obj in nodes:
            if obj in nodes_to_discard:
                continue
            node_type = tp.Dcc.node_type(obj)
            if node_type not in self._get_node_types() and not self._others_btn.isChecked():
                is_valid = False
                for node_type in self._get_node_types():
                    is_valid = tp.Dcc.check_object_type(obj, node_type, check_sub_types=True)
                    if is_valid:
                        break
                if not is_valid:
                    continue

            node_name = tp.Dcc.node_short_name(obj)
            item = QTreeWidgetItem(self._names_list, [node_name])
            item.obj = node_name
            item.preview_name = ''
            item.full_name = obj
            if tp.is_maya():
                mobj = maya.OpenMaya.MObject()
                sel = maya.OpenMaya.MSelectionList()
                sel.add(obj)
                sel.getDependNode(0, mobj)
                item.handle = maya.OpenMaya.MObjectHandle(mobj)

            self._names_list.addTopLevelItem(item)

    def _on_filter_names_changed(self, filter_text):
        """
        Internal callback function that is called each time the user enters text in the search line widget
        Shows or hides elements in the list taking in account the filter_text
        :param filter_text: str, current text
        """

        nodes_found = 0
        for i in range(self._names_list.topLevelItemCount()):
            item = self._names_list.topLevelItem(i)
            # item.setHidden(filter_text not in item.text(0))
            if filter_text not in item.text(0):
                item.setHidden(True)
            else:
                item.setHidden(False)
                nodes_found += 1

        if filter_text:
            self._search_lbl.setText('{} found'.format(nodes_found))
        else:
            self._search_lbl.setText('0 found')

    def _on_item_selection_changed(self):
        """
        Internal callback function that is triggered when the user selects an item in the names list
        """

        selected_items = self._names_list.selectedItems()
        item_names = [item.obj for item in selected_items]
        if len(selected_items) > 0:
            self._rename_btn.setText('Rename')
            self._rename_btn.setEnabled(True)
        else:
            self._rename_btn.setText('Select objects in the list to rename ...')
            self._rename_btn.setEnabled(False)

        # We reset name to original value
        for i in range(self._names_list.topLevelItemCount()):
            item = self._names_list.topLevelItem(i)
            if item.text(0) in item_names:
                continue
            if hasattr(item, 'obj'):
                item.setText(0, item.obj)

        self.doPreview.emit(selected_items, self._preview_btn.isChecked())

    def _on_select_all_clicked(self):
        for i in range(self._names_list.topLevelItemCount()):
            item = self._names_list.topLevelItem(i)
            item.setSelected(True)

    def _on_select_none_clicked(self):
        for i in range(self._names_list.topLevelItemCount()):
            item = self._names_list.topLevelItem(i)
            item.setSelected(False)

    def _on_refresh_list(self):
        self.doRefresh.emit(self)

    def _on_toggle_type(self, flag):
        self.doRefresh.emit(self)

    def _on_toggle_hide_default_scene_nodes_cbx(self, flag):
        self.doRefresh.emit(self)

    def _on_toggle_preview(self, flag):
        self.togglePreview.emit()


